function [Phi, convergence] = GLA(S, hop, N, Ncirc, w, its, satConv, ws,log)
% Function implementing Griffin and Lim Algorithm to generate a Phase from
% an STFT magnitude.
%
% Based on:
%   [1] 
%
% IN: 
%   S ... stft magnitude in a matrix format. Rows contain samples
%                     in time, columns samples in frequency.
%   hop ... Hop size for block processing
%   N ... block length
%	Ncirc ... circular shift length
%	w ... window function to use
%	its ... maximum number of iterations
%   satConv ... dB value for satisfying Convergence (stop Iterations)
%	ws ... window correction term 

% OUT:
%   Phi ... estimated phase values for the given magnitudes.
%	convergence ... for convergence plot
%
% Authors: Franz Zotter, Ruediger Fasching, Johanna Kerber, Korbinian Wegler
% Date: 2018

W = repmat(w,[1 size(S,2),size(S,3)]);
Nfft = size(S,1);
absS = abs(S);
Phi = angle(S);
convergence = zeros(its,1);

for it = 1:its
    S = absS.*exp(1i*Phi);        % construct complex exponential with (updated) phase
    S = real(ifft(S));            % inverse DFT (blocked) 
    S = circshift(S,Ncirc);       % shift the center of the window function to zero phase
    s = signal_overlapp_add(S,hop); % reconstructed signal  
    s  =  s./ws;
    S = signal_blocking(s,hop,N); % segment in blocks
    S = S(:,1:size(absS,2),:);       
    S = W.*S;
    S = [S;zeros(Nfft-N,size(S,2),size(S,3))]; % Zeropadding to DFT length
    S = circshift(S,-Ncirc);        
    S = fft(S,Nfft);              
    Phi = angle(S);               % new phase values
    absS_it = abs(S);
    convergence(it) = sum((absS_it(:)-absS(:)).^2)./sum(absS(:).^2);
    if abs(db(convergence(it))) > satConv
        break;
    end
end
fprintf(log,'Iterations: %d\n',it);
