function [magAVG, indexMaxMean] = average_magnitudes( ... 
												    magSTFT, ...
												    alpha ...
												    )
% Function to average over STFT magnitudes.
%
% IN: 
%   magSTFT ... Matrix of STFT magnitude vectors to be averaged.
%
%   alpha ... power of used averaging formula (1 ... arithmetic, inf ... max)
%
% OUT:
%   magAVG ... averaged STFT magnitude
%
%   indexMaxMean ... index of the loudest channel (maximum mean value)
%
% Authors: Ruediger Fasching, Johanna Kerber, Korbinian Wegler
% Date: 2018

numChannels = size(magSTFT, 3);

% find channel with maximum mean value over all bins and blocks 
[valueMaxMean, indexMaxMean] = max(mean(mean(abs(magSTFT))));

if alpha >= 1 && alpha < inf
	magAVG = (sum(magSTFT.^alpha, 3)/numChannels).^(1/alpha);
elseif alpha == inf % use maximum value for "averaging"
     magAVG = magSTFT(:,:,indexMaxMean);
elseif alpha == -1
  magAVG = (prod(magSTFT, 3)).^(1/numChannels); % geom
  fprintf('Geom. mean\n');
end
end 

% EOF