function S = signal_blocking(s, hop, N)
% Function segmenting a possibly multichannel signal for block processing.
%
% IN: 
%   s ... signal to segment
%
%   hop ... hopsize in Samples
%
%   N ... blocksize in Samples
%
% OUT:
%   S ... Matrix of segmented signal
%
% Authors: Franz Zotter, Ruediger Fasching, Johanna Kerber, Korbinian Wegler
% Date: 2018

L = size(s,1);
Hops = ceil(L/hop);
Ch = size(s,2);
S = zeros(N,Hops,Ch);
idx = 1:N;
maxhops1 = Hops-N/hop;

for h = 1:maxhops1
    S(:,h,:) = reshape(s((h-1)*hop+idx,:),[N 1 Ch]);    
end

for h = maxhops1+1:Hops
    lastidx = (h-1)*hop+idx;
    lastidx = lastidx(lastidx<=L);
    S(1:length(lastidx),h,:)=reshape(s(lastidx,:),[length(lastidx) 1 Ch]);
end